define("epi-ecf-ui/command/DeleteArchive", [
    // dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/promise/all",
    "dojo/topic",
    //dojox
    "dojox/widget/Standby",
    //cms
    "epi/dependency",
    "epi/shell/command/_Command",
    // Resources
    "epi/i18n!epi/cms/nls/commerce.widget.archive"
], function (
    array,
    declare,
    all,
    topic,
    Standby,
    dependency,
    _Command,
    res) {

    return declare([_Command], {
        // summary:
        //      A command that delete archived items when executed.
        // tags:
        //      public

        label: res.deletetooltip,

        iconClass: "epi-iconTrash",

        postscript: function () {
            this.inherited(arguments);

            var contentRepositoryDescriptors = dependency.resolve("epi.cms.contentRepositoryDescriptors");
            this.rootId = this.rootId || contentRepositoryDescriptors.catalog.roots[0];

            this._standby = new Standby({
                target: this.grid.domNode,
                color: "#fff",
                zIndex: 500,
                text: "<div class=\"oui-spinner\"></div>",
                centerIndicator: "text"
            }).placeAt(document.body);
            this._standby.startup();
        },

        _execute: function () {
            if (this.model) {
                this._showStandby(true);
                var archivedItems = this.model.get("selections");
                var store = this.model.store;
                var removePromises = [];

                if (!this.object) { // delete multiple
                    array.forEach(archivedItems, function (item) {
                        removePromises.push(store.remove(item.contentLink));
                    });
                } else { // delete single
                    removePromises.push(store.remove(this.object.contentLink));
                }

                all(removePromises).then(function () {
                    this.grid.clearSelection();
                    this._viewArchive();
                    this._showStandby(false);
                }.bind(this));
            }
        },

        _showStandby: function (visible) {
            // summary:
            //      Set standby visibility.
            // tags:
            //      private

            if (!this._standby) {
                return;
            }
            if (visible) {
                if (!this._standby.isVisible()) {
                    this._standby.show();
                }
            } else {
                this._standby.hide();
            }
        },
        
        _onModelChange: function () {
            this.set("canExecute", true);
            this.set("isAvailable", true);
        },

        _viewArchive: function () {
            topic.publish("/epi/shell/context/request", { uri: "epi.cms.contentdata:///" + this.rootId }, { viewName: "archivelistview" });
        }
    });
});
