define("epi-ecf-ui/command/MoveToArchive", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/when",
    "dojo/on",
    "dojo/topic",

    //dojox
    "dojox/widget/Standby",
    
    //cms
    "epi/dependency",
    "epi/shell/command/_Command",
    "epi/shell/command/_SelectionCommandMixin",
    "epi/shell/TypeDescriptorManager",
    "epi/shell/widget/dialog/Alert",
    "epi/shell/widget/dialog/Dialog",
    "epi/string",
    "epi-cms/_ContentContextMixin",
    "epi-cms/core/ContentReference",
    "epi-cms/widget/ContentReferences",
    // epi-ecf-ui
    "../contentediting/ModelSupport",
    // Resources
    "epi/i18n!epi/cms/nls/commerce.command.movetoarchive"
], function (
    array,
    declare, 
    when,
    on,
    topic,

    Standby,

    dependency,
    _Command,
    _SelectionCommandMixin,
    TypeDescriptorManager,
    Alert,
    Dialog,
    epiString,
    _ContentContextMixin,
    ContentReference,
    
    ContentReferences,

    ModelSupport,

    res) {

    return declare([_Command, _ContentContextMixin, _SelectionCommandMixin], {
        // summary:
        //      A command that move catalog contents to archive.
        // tags:
        //      public

        // label: [readonly] String
        //		The action text of the command to be used in visual elements.
        label: res.label,

        contents: [],

        contentName: "",

        // iconClass: [readonly] String
        //		The icon class of the command to be used in visual elements.
        iconClass: "epi-iconArchive",

        storeKey: "epi.commerce.archive",
        
        postscript: function () {
            this.inherited(arguments);
            var registry = dependency.resolve("epi.storeregistry");
            this.store = this.store || registry.get(this.storeKey);

            var contentRepositoryDescriptors = dependency.resolve("epi.cms.contentRepositoryDescriptors");
            this.rootId = contentRepositoryDescriptors.catalog.roots[0];

            this._standby = new Standby({
                target: document.body,
                color: "#fff",
                zIndex: 500,
                text: "<div class=\"oui-spinner\"></div>",
                centerIndicator: "text"
            }).placeAt(document.body);
            this._standby.startup();
        },

        _execute: function () {
            // summary:
            //		Executes to move selected catalog contents to archive.
            // tags:
            //		protected

            var isShowDialog = false;
            var name = "";

            this.contentName = "";
            this.contents = [];
            
            array.forEach(this.selection.data, function(x) {
                this.contents.push({ 
                    contentLink: x.data.contentLink,
                    parentLink: x.data.parentLink,
                    typeIdentifier: x.data.typeIdentifier
                 });
                name = name.length > 0 ? name + ", " + x.data.name : x.data.name;
                if (!isShowDialog) {
                    isShowDialog = x.data.hasChildren || TypeDescriptorManager.isBaseTypeIdentifier(x.data.typeIdentifier, ModelSupport.contentTypeIdentifier.nodeContentBase);
                }
            }.bind(this));

            this.contentName = name;
            if (this.contents.length > 0) {
                if (!isShowDialog) {
                    this._moveToArchive();
                } else {
                    var content = new ContentReferences({
                        mode: "show",
                        contentItems: this.contents,
                        storeKeyName: "epi.commerce.catalogreferenced",
                        onContextChanged: function () {}
                    });

                    var watch = content.watch("numberOfReferences", function (propertyName, oldValue, newValue) {

                        // Remove the watch so we don't leak memory or create additional dialogs on refresh.
                        watch.remove();

                        var dialog = new Dialog({
                            dialogClass: "epi-dialog-contentReferences",
                            closeIconVisible: true,
                            focusActionsOnLoad: true,
                            cancelActionText: res.cancel,
                            confirmActionText: res.confirm,
                            title: res.confirmpopuptitle,
                            description: res.confirmpopupcontent
                        });
                        dialog.connect(dialog, "onExecute", this._moveToArchive.bind(this));
                        dialog.set("content", content);

                        var handle = content.on("viewReference", function () {
                            dialog.hide();
                            handle.remove();
                        });
                        var refreshHandle = content.grid.on("dgrid-refresh-complete", function () {
                            refreshHandle.remove();
                            dialog.show();
                        }.bind(this));
                    }.bind(this));

                    content.startup();
                }
            }
        },

        _moveToArchive: function () {
            // tags: private

            this._showStandby(true);
            when(this.store.add({ contents: this.contents, action: ModelSupport.archivedAction.move }),
                function (response) {
                    var message = '"' + this.contentName + '"&nbsp;' + res.undomessage + '&nbsp;<a class="epi-archive-undo">' + res.undo + "</a>";
                    this._showNotification(message, response);
                    this._refresh();
                    this._showStandby(false);
                }.bind(this),
                function (errorResponse) {
                    var alertDialog = new Alert({
                        description: errorResponse
                    });

                    alertDialog.show();
                }
            );
        },

        _showStandby: function (visible) {
            // summary:
            //      Set standby visibility.
            // tags:
            //      private

            if (!this._standby) {
                return;
            }
            if (visible) {
                if (!this._standby.isVisible()) {
                    this._standby.show();
                }
            } else {
                this._standby.hide();
            }
        },

        _showNotification: function (message, contents) {
            var alertDialog = new Alert({
                title: epiString.toHTML(message),
                dialogClass: "archive-notification",
                closeIconVisible: true
            });
            alertDialog.show();
            var wrapper = dojo.query(".dijitDialogUnderlayWrapper");
            if (wrapper) {
                wrapper.addClass("hide");
                setTimeout(function () {
                    alertDialog.hide();
                    wrapper.removeClass("hide");
                }.bind(this), 2000);
            }

            var undoElement = dojo.query(".epi-archive-undo");
            if (undoElement) {
                on(undoElement, "click", function (evt) {
                    this.store.put({ Contents: contents, action: ModelSupport.archivedAction.restore }).then(function () {
                        this._refresh();
                    }.bind(this));
                    alertDialog.hide();
                    wrapper.removeClass("hide");
                }.bind(this));
            }
        },

        _refresh: function () {
            when(this.getCurrentContent()).then(function (content) {
                if (array.some(this.contents, function (item) {
                    return ContentReference.compareIgnoreVersion(item.contentLink, content.contentLink);
                })) {
                    topic.publish("/epi/shell/context/request", { uri: "epi.cms.contentdata:///" + content.parentLink }, { sender: this });
                } else {
                    
                    topic.publish("/epi/shell/context/request", { uri: "epi.cms.contentdata:///" + this.rootId }, { sender: this });
                }
            }.bind(this));
        },
        
        _onModelChange: function () {
            // summary:
            //		Updates canExecute after the model has been updated.
            // tags:
            //		protected

            this.set("canExecute", false);
            if (this.selection.data.length) {
                this.set("canExecute", true);
            }

            var isAvailable = 
                this.model.typeIdentifier !== ModelSupport.contentTypeIdentifier.rootContent && 
                this.model.typeIdentifier !== ModelSupport.contentTypeIdentifier.catalogContent &&
                this.model.parentLink !== this.rootId;

            this.set("isAvailable", isAvailable);

            this.contentName = "";
            this.contents = [];
        }
    });
});
