define("epi-ecf-ui/command/RestoreArchive", [
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojo/topic",
    "dojo/when",

    //dojox
    "dojox/widget/Standby",

    //cms
    "epi/dependency",
    "epi/string",
    "epi/shell/command/_Command",
    "epi/shell/widget/dialog/Dialog",
    "../contentediting/ModelSupport",
    "../widget/CatalogContentSelectorDialog",

    // Resources
    "epi/i18n!epi/cms/nls/commerce.widget.archive"
], function (
    array,
    declare,
    lang,
    Deferred,
    topic,
    when,

    //dojox
    Standby,

    //cms
    dependency,
    epistring,
    _Command,
    Dialog,
    ModelSupport,
    CatalogContentSelectorDialog,

    //resource
    res) {

    return declare([_Command], {
        // summary:
        //      A command that view archive list when executed.
        //
        // tags:
        //      public

        // label: [readonly] String
        //		The action text of the command to be used in visual elements.
        label: res.restoretooltip,

        // iconClass: [readonly] String
        //		The icon class of the command to be used in visual elements.
        iconClass: "epi-iconRevert",

        storeKey: "epi.commerce.archive",

        rootId: 0,

        postscript: function () {
            this.inherited(arguments);
            var registry = dependency.resolve("epi.storeregistry");
            this._store = this._store || registry.get(this.storeKey);

            var contentRepositoryDescriptors = dependency.resolve("epi.cms.contentRepositoryDescriptors");
            this.rootId = contentRepositoryDescriptors.catalog.roots[0];

            this._standby = new Standby({
                target: this.grid.domNode,
                color: "#fff",
                zIndex: 500,
                text: "<div class=\"oui-spinner\"></div>",
                centerIndicator: "text"
            }).placeAt(document.body);
            this._standby.startup();
        },

        _execute: function () {
            // summary:
            //		Restore Archive
            // tags:
            //		protected
            if (this.model && this.model.get("selections")) {
                this._showStandby(true);
                var store = this.model.store;
                var archiveItems = this.model.get("selections");
                // restore multiple
                if (archiveItems.length > 0 && !this.object) {
                    return when(this._restoreWithNewParent(archiveItems), lang.hitch(this, this._restoreCompleted));
                }

                // restore single
                //this._showStandby(true);
                archiveItems = [this.object];
                var getRestorePoint = {
                    archivedId: this.object.archivedId,
                    query: "getrestorepoint"
                };
                if (this.object.isArchived) {
                    //this._showStandby(true);
                    store.query(getRestorePoint).then(function (contentLink) {
                        if (contentLink) {
                            when(this._restoreArchivedItem(archiveItems), lang.hitch(this, this._restoreCompleted));
                        } else {
                            when(this._restoreWithNewParent(archiveItems), lang.hitch(this, this._restoreCompleted));
                        }
                    }.bind(this));
                } else {
                    when(this._restoreWithNewParent(archiveItems), lang.hitch(this, this._restoreCompleted));
                }
            }
        },

        _showStandby: function (visible) {
            // summary:
            //      Set standby visibility.
            // tags:
            //      private

            if (!this._standby) {
                return;
            }
            if (visible) {
                if (!this._standby.isVisible()) {
                    this._standby.show();
                }
            } else {
                this._standby.hide();
            }
        },

        _restoreCompleted: function () {
            this._showStandby(false);
            this.grid.clearSelection();
            this._viewArchive();
        },

        _onModelChange: function () {
            this.set("canExecute", true);
            this.set("isAvailable", true);
        },

        _restoreWithNewParent: function(archiveItems) {
            var df = new Deferred();

            this._selectNewParent().then(function (contentLink) {
                this._restoreArchivedItem(archiveItems, contentLink).then(df.resolve);
            }.bind(this));

            return df.promise;
        },

        _selectNewParent: function () {
            var df = new Deferred(),
                settings = {
                    showButtons: false,
                    roots: [this.rootId],
                    canSelectOwnerContent: false,
                    allowedTypes: [ModelSupport.contentTypeIdentifier.catalogContentBase, ModelSupport.contentTypeIdentifier.nodeContentBase],
                    restrictedTypes: [ModelSupport.contentTypeIdentifier.rootContent, ModelSupport.contentTypeIdentifier.entryContentBase],
                    showAllLanguages: true
                };

            var catalogSelector = new CatalogContentSelectorDialog(settings);

            var dialog = new Dialog({
                title: epistring.toHTML(res.newparenttitle),
                description: epistring.toHTML(res.newparentdes),
                confirmActionText: epistring.toHTML(res.newparentselect),
                content: catalogSelector
            });

            dialog.on("execute", function() {
                var contentLink = catalogSelector.get("value");
                df.resolve(contentLink);
            });

            dialog.on("cancel", function () { this._showStandby(false); }.bind(this));

            dialog.show();
            this._setDisableSelectNewParent(dialog, true);
            catalogSelector.onChange = function () {
                this._setDisableSelectNewParent(dialog, true);
                if (catalogSelector.get("value") !== '') {
                    this._setDisableSelectNewParent(dialog, false);
                }
            }.bind(this);

            return df.promise;
        },

        _setDisableSelectNewParent: function (dialog, value) {
            dialog.definitionConsumer.setItemProperty(dialog._okButtonName, "disabled", value);
        },

        _restoreArchivedItem: function(archiveItems, newParentLink) {
            var df = new Deferred(),
                contents = [],
                store = this.model.store;
                
            array.forEach(archiveItems, function(item) {
                contents.push({ contentLink: item.contentLink, parentLink: newParentLink });
            });
            var param = {
                Contents: contents,
                action: ModelSupport.archivedAction.restore
            };
            store.put(param).then(function () {
                df.resolve();
            });

            return df.promise;
        },

        _viewArchive: function () {
            topic.publish("/epi/shell/context/request", { uri: "epi.cms.contentdata:///" + this.rootId }, { viewName: "archivelistview" });
        }
    });
});
