﻿define("epi-ecf-ui/component/CatalogNavigationTree", [
// dojo
    "dojo/_base/declare",
    "dojo/Deferred",
    "dojo/when",
    "dijit/form/Button",
// epi
    "epi/shell/command/_CommandModelBindingMixin",
    "epi-cms/component/ContentNavigationTree",
// commerce
    "../command/ViewArchive",
    "../widget/viewmodel/CatalogTreeStoreModel",
    "./ContentContextMenuCommandProvider",
    "./commerceDndSource"
],

function (
// dojo
    declare,
    Deferred,
    when,

    Button,
// epi
    _CommandModelBindingMixin,
    ContentNavigationTree,
// commerce
    ViewArchiveCommand,
    CatalogTreeStoreModel,
    ContextMenuCommandProvider,
    commerceDndSource
 ) {

    // module:
    //      epi-ecf-ui.component.CatalogNavigationTree

    return declare([ContentNavigationTree], {
        // summary:
        //      Catalog tree component, which support market filtering.
        // tags:
        //      public

        _buttonClass: declare([Button, _CommandModelBindingMixin]),

        dndSource: commerceDndSource,

        contextMenuCommandProvider: ContextMenuCommandProvider,

        startup: function () {
            this.inherited(arguments);

            var consumer = this.getConsumer();
            if (!consumer) {
                return;
            }

            if (consumer.toolbar && consumer.toolbar.domNode) {
                var viewArchiveCommand = new ViewArchiveCommand({
                    model: true
                });
                var viewArchiveWidget = new this._buttonClass({
                    model: viewArchiveCommand,
                    "class": "epi-componentSettingsButton epi-componentToolbarButton epi-mediumButton",
                    showLabel: false
                });
                viewArchiveWidget._command = viewArchiveCommand;

                viewArchiveWidget.placeAt(consumer.toolbar.domNode);
            }
        },

        _createTreeModel: function () {
            // summary:
            //      Create content tree model
            // tags:
            //      protected override

            return new CatalogTreeStoreModel({
                roots: this.roots,
                typeIdentifiers: this.typeIdentifiers,
                containedTypes: this.settings.containedTypes
            });
        },

        // Remark: Because cms-ui starts to use confirm on pasteItems and not just pasteItem
        _wrapModel: function (model) {
            return model;
        },

        _updateGlobalToolbarButtons: function (targetNode) {

            if (targetNode){
                var parent = targetNode.getParent();

                targetNode.item.contextId = parent.dndData.contentLink;
            }

            this.inherited(arguments);
        },

        selectContent: function (contentReferenceAsString, setFocus, needParentRefresh, onComplete) {
            // summary:
            //      Select a given content.
            // tags:
            //      public override

            var args = arguments,
                inherited = this.getInherited(args),
                deferred = new Deferred();

            return when(this.model.store.get(contentReferenceAsString), function (content) {
                if (content && !content.properties.isLinkedToCurrentCategory) {
                    return inherited.apply(this, args);
                }

                return deferred.promise;
            }.bind(this));
        }
    });

});